/*

Location: TempDB
Location scope: ##Global or #Local
                Global are accessible from anywhere
                Local are accessible from the connection that created them
Time scope: Exist until dropped or until creating connection is terminated
Indexable: Yes (Clustered and Nonclustered)
Copy of data: Yes
Modifiable: Yes, modifies copy of data

*/

-- Basic Temp Table Usage

/*
create table ##JK_Test_Temp_Table (
       myIntKey int not null,
       myVarchar varchar(10) not null
CONSTRAINT [PK_JK_Test_Temp_Table] PRIMARY KEY CLUSTERED 
(
	[myIntKey] ASC
)
);
*/

/*
insert into ##JK_Test_Temp_Table (myIntKey, myVarchar) 
values (1, 'A'),
       (2, 'B'),
       (3, 'A'),
       (4, 'C');
*/

-- You can create indexes after the fact on temp tables
-- Note the change in execution plan

/*
-- Enable Execution Plan
select *
  from ##JK_Test_Temp_Table
where myVarchar = 'A';
*/

/*
-- Disable Execution Plan
create nonclustered index JK_Test_Temp_Table_SIDX_1 on ##JK_Test_Temp_Table(
       myVarchar asc
);
*/

/*
-- Enable Execution Plan
select *
  from ##JK_Test_Temp_Table
where myVarchar = 'A';
*/

-- You can update a temp table

/*
update tgt
   set myVarchar = 'D'
  from ##JK_Test_Temp_Table tgt
 where myInt = 4;

select *
  from ##JK_Test_Temp_Table;
*/

-- Global versus local

/*
create table #JK_Test_Temp_Table_2 (
       myIntKey int not null,
       myVarchar varchar(10) not null
CONSTRAINT [PK_JK_Test_Temp_Table_2] PRIMARY KEY CLUSTERED 
(
	[myIntKey] ASC
)
);
*/

/*
insert into #JK_Test_Temp_Table_2 (myIntKey, myVarchar) 
values (1, 'A'),
       (2, 'B'),
       (3, 'A'),
       (4, 'C');
*/

/*
select *
  from #JK_Test_Temp_Table_2
where myVarchar = 'A';
*/

-- Temp tables go away when nothing is using them
-- Run this after closing the window you created the temp table in

/*
select *
  from ##JK_Test_Temp_Table
where myVarchar = 'A';
*/

-- Modifying the temp table is independent of the data that populated it

/*
select *
  into #JK_Test_Temp_Table_3
 from dbo.JK_Temp_Stuff_Test_Table;

update #JK_Test_Temp_Table_3
   set myVarchar = 'Whooo';

select * from #JK_Test_Temp_Table_3;
select * from dbo.JK_Temp_Stuff_Test_Table;
*/
